@@ -636,7 +636,7 @@ def linkage_testfile(self, lgdict, popt, desc = ''):
     if '' != desc:
         desc = desc + '-'
     testfile = clg.test_data_srcdir + "parses-" + desc + clg.dictionary_get_lang(lgdict._obj) + ".txt"
-    parses = open(testfile)
+    parses = open(testfile, "rb")
     diagram = None
     sent = None
     lineno = 0
