@@ -19,6 +19,8 @@
  * - int is >= 32 bit (for random number)
  */
 
+#include "utilities.h" /* included first, for MSVC rand_s() */
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -32,10 +34,11 @@
 #include "regex-morph.h"
 #include "structures.h"
 #include "tokenize.h"
-#include "utilities.h"
 
 #include "anysplit.h"
 
+#define D_AS 10          /* verbosity level for this file */
+
 extern const char * const afdict_classname[];
 
 typedef int p_start;     /* partition start in a word */
@@ -350,8 +353,7 @@ bool anysplit_init(Dictionary afdict)
 
 	if (0 == regparts->length)
 	{
-		/* FIXME: Early assignment of verbosity by -v=x argument. */
-		if (verbosity > 1)
+		if (debug_level(+D_AS))
 			prt_error("Warning: File %s: Anysplit disabled (%s not defined)",
 		             afdict->name, afdict_classname[AFDICT_REGPARTS]);
 		return true;
