@@ -77,7 +77,7 @@ struct prune_context_s
 	bool null_links;
 	int power_cost;
 	int N_changed;   /* counts the number of changes
-						   of c->word fields in a pass */
+						   of c->nearest_word fields in a pass */
 	power_table *pt;
 	Sentence sent;
 };
@@ -854,15 +854,6 @@ static void put_into_power_table(unsigned int size, C_list ** t, Connector * c,
 	m->shallow = shal;
 }
 
-static int set_dist_fields(Connector * c, size_t w, int delta)
-{
-	int i;
-	if (c == NULL) return (int) w;
-	i = set_dist_fields(c->next, w, delta) + delta;
-	c->word = i;
-	return i;
-}
-
 /**
  * Allocates and builds the initial power hash tables
  */
@@ -872,7 +863,7 @@ static power_table * power_table_new(Sentence sent)
 	size_t w, len;
 	unsigned int i, size;
 	C_list ** t;
-	Disjunct * d, * xd, * head;
+	Disjunct * d;
 	Connector * c;
 
 	pt = (power_table *) xalloc (sizeof(power_table));
@@ -882,35 +873,16 @@ static power_table * power_table_new(Sentence sent)
 	pt->l_table = xalloc (2 * sent->length * sizeof(C_list **));
 	pt->r_table = pt->l_table + sent->length;
 
-   /* First, we initialize the word fields of the connectors, and
-	  eliminate those disjuncts with illegal connectors */
-	for (w=0; w<sent->length; w++)
-	{
-	  head = NULL;
-	  for (d=sent->word[w].d; d!=NULL; d=xd) {
-		  xd = d->next;
-		  if ((set_dist_fields(d->left, w, -1) < 0) ||
-			  (set_dist_fields(d->right, w, 1) >= (int) sent->length)) {
-			  d->next = NULL;
-			  free_disjuncts(d);
-		  } else {
-			  d->next = head;
-			  head = d;
-		  }
-	  }
-	  sent->word[w].d = head;
-	}
-
 	for (w=0; w<sent->length; w++)
 	{
 		/* The below uses variable-sized hash tables. This seems to
 		 * provide performance that is equal or better than the best
-		 * fixed-size prformance.
+		 * fixed-size performance.
 		 * The best fixed-size performance seems to come at about
 		 * a 1K table size, for both English and Russian. (Both have
 		 * about 100 fixed link-types, and many thousands of auto-genned
 		 * link types (IDxxx idioms for both, LLxxx suffix links for
-		 * Russian).  Plusses and minuses:
+		 * Russian).  Pluses and minuses:
 		 * + small fixed tables are faster to initialize.
 		 * - small fixed tables have more collisions
 		 * - variable-size tables require counting connectors.
@@ -930,7 +902,7 @@ static power_table * power_table_new(Sentence sent)
 			c = d->left;
 			if (c != NULL) {
 				put_into_power_table(size, t, c, true);
-				for (c=c->next; c!=NULL; c=c->next){
+				for (c=c->next; c!=NULL; c=c->next) {
 					put_into_power_table(size, t, c, false);
 				}
 			}
@@ -970,7 +942,7 @@ static void clean_table(unsigned int size, C_list ** t)
 		head = NULL;
 		for (m = t[i]; m != NULL; m = xm) {
 			xm = m->next;
-			if (m->c->word != BAD_WORD) {
+			if (m->c->nearest_word != BAD_WORD) {
 				m->next = head;
 				head = m;
 			} else {
@@ -986,16 +958,16 @@ static void clean_table(unsigned int size, C_list ** t)
  * (and the two words that these came from) and returns TRUE if it is
  * possible for these two to match based on local considerations.
  */
-static bool possible_connection(prune_context *pc,
-                               Connector *lc, Connector *rc,
-                               bool lshallow, bool rshallow,
-                               int lword, int rword)
+static bool possible_connection(bool no_null_links,
+                                Connector *lc, Connector *rc,
+                                bool lshallow, bool rshallow,
+                                int lword, int rword)
 {
 	int dist;
 	if ((!lshallow) && (!rshallow)) return false;
 
 	/* Two deep connectors can't work */
-	if ((lc->word > rword) || (rc->word < lword)) return false;
+	if ((lc->nearest_word > rword) || (rc->nearest_word < lword)) return false;
 
 	dist = rword - lword;
 	// assert(0 < dist, "Bad word order in possible connection.");
@@ -1003,7 +975,7 @@ static bool possible_connection(prune_context *pc,
 	/* Word range constraints */
 	if (1 == dist)
 	{
-		if (!((lc->next == NULL) && (rc->next == NULL))) return false;
+		if ((lc->next != NULL) || (rc->next != NULL)) return false;
 	}
 	else
 	if (dist > lc->length_limit || dist > rc->length_limit)
@@ -1017,7 +989,7 @@ static bool possible_connection(prune_context *pc,
 	 * i.e. before well allow null-linked words.
 	 */
 	else
-	if ((!pc->null_links) &&
+	if (no_null_links &&
 	    (lc->next == NULL) &&
 	    (rc->next == NULL) &&
 	    (!lc->multi) && (!rc->multi))
@@ -1033,22 +1005,18 @@ static bool possible_connection(prune_context *pc,
  * a connector that can match to c.  shallow tells if c is shallow.
  */
 static bool
-right_table_search(prune_context *pc, int w, Connector *c,
+right_table_search(bool nonul, power_table *pt, int w, Connector *c,
                    bool shallow, int word_c)
 {
 	unsigned int size, h;
 	C_list *cl;
-	power_table *pt;
 
-	pt = pc->pt;
 	size = pt->r_table_size[w];
 	h = connector_hash(c) & (size-1);
 	for (cl = pt->r_table[w][h]; cl != NULL; cl = cl->next)
 	{
-		if (possible_connection(pc, cl->c, c, cl->shallow, shallow, w, word_c))
-		{
+		if (possible_connection(nonul, cl->c, c, cl->shallow, shallow, w, word_c))
 			return true;
-		}
 	}
 	return false;
 }
@@ -1058,19 +1026,17 @@ right_table_search(prune_context *pc, int w, Connector *c,
  * a connector that can match to c.  shallows tells if c is shallow
  */
 static bool
-left_table_search(prune_context *pc, int w, Connector *c,
+left_table_search(bool nonul, power_table *pt, int w, Connector *c,
                   bool shallow, int word_c)
 {
 	unsigned int size, h;
 	C_list *cl;
-	power_table *pt;
 
-	pt = pc->pt;
 	size = pt->l_table_size[w];
 	h = connector_hash(c) & (size-1);
 	for (cl = pt->l_table[w][h]; cl != NULL; cl = cl->next)
 	{
-		if (possible_connection(pc, c, cl->c, shallow, cl->shallow, word_c, w))
+		if (possible_connection(nonul, c, cl->c, shallow, cl->shallow, word_c, w))
 			return true;
 	}
 	return false;
@@ -1081,7 +1047,7 @@ left_table_search(prune_context *pc, int w, Connector *c,
  * w-1, w-2, w-3...  Returns the word to which the first connector of
  * the list could possibly be matched.  If c is NULL, returns w.  If
  * there is no way to match this list, it returns a negative number.
- * If it does find a way to match it, it updates the c->word fields
+ * If it does find a way to match it, it updates the c->nearest_word fields
  * correctly.
  */
 static int
@@ -1089,11 +1055,11 @@ left_connector_list_update(prune_context *pc, Connector *c,
                            int w, bool shallow)
 {
 	int n, lb;
-	bool foundmatch;
+	bool foundmatch, no_null_links;
 
 	if (c == NULL) return w;
 	n = left_connector_list_update(pc, c->next, w, false) - 1;
-	if (((int) c->word) < n) n = c->word;
+	if (((int) c->nearest_word) < n) n = c->nearest_word;
 
 	/* lb is now the leftmost word we need to check */
 	lb = w - c->length_limit;
@@ -1101,18 +1067,19 @@ left_connector_list_update(prune_context *pc, Connector *c,
 
 	/* n is now the rightmost word we need to check */
 	foundmatch = false;
+	no_null_links = !pc->null_links;
 	for (; n >= lb ; n--)
 	{
 		pc->power_cost++;
-		if (right_table_search(pc, n, c, shallow, w))
+		if (right_table_search(no_null_links, pc->pt, n, c, shallow, w))
 		{
 			foundmatch = true;
 			break;
 		}
 	}
-	if (n < ((int) c->word))
+	if (n < ((int) c->nearest_word))
 	{
-		c->word = n;
+		c->nearest_word = n;
 		pc->N_changed++;
 	}
 	return (foundmatch ? n : -1);
@@ -1124,36 +1091,37 @@ left_connector_list_update(prune_context *pc, Connector *c,
  * the list could possibly be matched.  If c is NULL, returns w.  If
  * there is no way to match this list, it returns a number greater than
  * N_words - 1.   If it does find a way to match it, it updates the
- * c->word fields correctly.
+ * c->nearest_word fields correctly.
  */
 static size_t
 right_connector_list_update(prune_context *pc, Sentence sent, Connector *c,
                             size_t w, bool shallow)
 {
 	size_t n, ub;
-	bool foundmatch;
+	bool foundmatch, no_null_links;
 
 	if (c == NULL) return w;
 	n = right_connector_list_update(pc, sent, c->next, w, false) + 1;
-	if (c->word > n) n = c->word;
+	if (c->nearest_word > n) n = c->nearest_word;
 
 	/* ub is now the rightmost word we need to check */
 	ub = w + c->length_limit;
-	if (ub > sent->length) ub = sent->length;
+	if (ub > sent->length) ub = sent->length - 1;
 
 	/* n is now the leftmost word we need to check */
 	foundmatch = false;
+	no_null_links = !pc->null_links;
 	for (; n <= ub ; n++)
 	{
 		pc->power_cost++;
-		if (left_table_search(pc, n, c, shallow, w))
+		if (left_table_search(no_null_links, pc->pt, n, c, shallow, w))
 		{
 			foundmatch = true;
 			break;
 		}
 	}
-	if (n > c->word) {
-		c->word = n;
+	if (n > c->nearest_word) {
+		c->nearest_word = n;
 		pc->N_changed++;
 	}
 	return (foundmatch ? n : sent->length);
@@ -1189,12 +1157,11 @@ int power_prune(Sentence sent, Parse_Options opts)
 
 		/* left-to-right pass */
 		for (w = 0; w < sent->length; w++) {
-			if ((2 == w%7) && parse_options_resources_exhausted(opts)) break;
 			for (d = sent->word[w].d; d != NULL; d = d->next) {
 				if (d->left == NULL) continue;
 				if (left_connector_list_update(pc, d->left, w, true) < 0) {
-					for (c=d->left;  c != NULL; c = c->next) c->word = BAD_WORD;
-					for (c=d->right; c != NULL; c = c->next) c->word = BAD_WORD;
+					for (c=d->left;  c != NULL; c = c->next) c->nearest_word = BAD_WORD;
+					for (c=d->right; c != NULL; c = c->next) c->nearest_word = BAD_WORD;
 					N_deleted++;
 					total_deleted++;
 				}
@@ -1204,7 +1171,7 @@ int power_prune(Sentence sent, Parse_Options opts)
 			nd = NULL;
 			for (d = sent->word[w].d; d != NULL; d = dx) {
 				dx = d->next;
-				if ((d->left != NULL) && (d->left->word == BAD_WORD)) {
+				if ((d->left != NULL) && (d->left->nearest_word == BAD_WORD)) {
 					d->next = free_later;
 					free_later = d;
 				} else {
@@ -1225,12 +1192,11 @@ int power_prune(Sentence sent, Parse_Options opts)
 		/* right-to-left pass */
 
 		for (w = sent->length-1; w != (size_t) -1; w--) {
-			if ((2 == w%7) && parse_options_resources_exhausted(opts)) break;
 			for (d = sent->word[w].d; d != NULL; d = d->next) {
 				if (d->right == NULL) continue;
 				if (right_connector_list_update(pc, sent, d->right, w, true) >= sent->length) {
-					for (c=d->right; c != NULL; c = c->next) c->word = BAD_WORD;
-					for (c=d->left;  c != NULL; c = c->next) c->word = BAD_WORD;
+					for (c=d->right; c != NULL; c = c->next) c->nearest_word = BAD_WORD;
+					for (c=d->left;  c != NULL; c = c->next) c->nearest_word = BAD_WORD;
 					N_deleted++;
 					total_deleted++;
 				}
@@ -1239,7 +1205,7 @@ int power_prune(Sentence sent, Parse_Options opts)
 			nd = NULL;
 			for (d = sent->word[w].d; d != NULL; d = dx) {
 				dx = d->next;
-				if ((d->right != NULL) && (d->right->word == BAD_WORD)) {
+				if ((d->right != NULL) && (d->right->nearest_word == BAD_WORD)) {
 					d->next = free_later;
 					free_later = d;
 				} else {
@@ -1520,6 +1486,7 @@ static int pp_prune(Sentence sent, Parse_Options opts)
 	multiset_table *cmt;
 
 	if (sent->postprocessor == NULL) return 0;
+	if (!opts->perform_pp_prune) return 0;
 
 	knowledge = sent->postprocessor->knowledge;
 
@@ -1634,11 +1601,18 @@ static int pp_prune(Sentence sent, Parse_Options opts)
 void pp_and_power_prune(Sentence sent, Parse_Options opts)
 {
 	power_prune(sent, opts);
+	pp_prune(sent, opts);
 
+	return;
+
+	// Not reached. We can actually gain a few percent of
+	// performance be skipping the loop below. Mostly, it just
+	// does a lot of work, and pretty much finds nothing.
+	// And so we skip it.
+#ifdef ONLY_IF_YOU_THINK_THIS_IS_WORTH_IT
 	for (;;) {
-		if (parse_options_resources_exhausted(opts)) break;
 		if (pp_prune(sent, opts) == 0) break;
-		if (parse_options_resources_exhausted(opts)) break;
 		if (power_prune(sent, opts) == 0) break;
 	}
+#endif
 }
